/*
 * Copyright 1999-2101 Alibaba Group Holding Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.alibaba.druid.bvt.sql.mysql;

import java.util.List;

import org.junit.Assert;

import com.alibaba.druid.sql.MysqlTest;
import com.alibaba.druid.sql.SQLUtils;
import com.alibaba.druid.sql.ast.SQLStatement;
import com.alibaba.druid.sql.dialect.mysql.parser.MySqlStatementParser;
import com.alibaba.druid.sql.dialect.mysql.visitor.MySqlSchemaStatVisitor;
import com.alibaba.druid.stat.TableStat;

public class MySqlCreateTableTest42 extends MysqlTest {

    public void test_0() throws Exception {
        String sql = "CREATE TABLE rc (" + //
                     "    a INT NOT NULL, " + //
                     "    b INT NOT NULL" + //
                     ")" + //
                     "PARTITION BY RANGE COLUMNS(a,b) (" + //
                     "    PARTITION p0 VALUES LESS THAN (10,5)," + //
                     "    PARTITION p1 VALUES LESS THAN (20,10)," + //
                     "    PARTITION p2 VALUES LESS THAN (MAXVALUE,15)," + //
                     "    PARTITION p3 VALUES LESS THAN (MAXVALUE,MAXVALUE)" + //
                     ");"; //

        MySqlStatementParser parser = new MySqlStatementParser(sql);
        List<SQLStatement> statementList = parser.parseStatementList();
        SQLStatement stmt = statementList.get(0);
        print(statementList);

        Assert.assertEquals(1, statementList.size());

        MySqlSchemaStatVisitor visitor = new MySqlSchemaStatVisitor();
        stmt.accept(visitor);

        System.out.println("Tables : " + visitor.getTables());
        System.out.println("fields : " + visitor.getColumns());
        System.out.println("coditions : " + visitor.getConditions());
        System.out.println("orderBy : " + visitor.getOrderByColumns());

        Assert.assertEquals(1, visitor.getTables().size());
        Assert.assertEquals(2, visitor.getColumns().size());
        Assert.assertEquals(0, visitor.getConditions().size());

        Assert.assertTrue(visitor.getTables().containsKey(new TableStat.Name("rc")));

        String output = SQLUtils.toMySqlString(stmt);
        Assert.assertEquals("CREATE TABLE rc (" + //
                            "\n\ta INT NOT NULL, " + //
                            "\n\tb INT NOT NULL" + //
                            "\n)"
                            + "\nPARTITION BY RANGE COLUMNS (a, b)"
                            + "\n(" + //
                            "\n\tPARTITION p0 VALUES LESS THAN (10, 5)," + //
                            "\n\tPARTITION p1 VALUES LESS THAN (20, 10)," + //
                            "\n\tPARTITION p2 VALUES LESS THAN (MAXVALUE, 15)," + //
                            "\n\tPARTITION p3 VALUES LESS THAN (MAXVALUE, MAXVALUE)" + //
                            "\n)", output);

    }
}
